@@ -1,16 +1,16 @@
-# =========== Copyright 2023 @ CAMEL-AI.org. All Rights Reserved. ===========
-# Licensed under the Apache License, Version 2.0 (the “License”);
+# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
+# Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
 #     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an “AS IS” BASIS,
+# distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-# =========== Copyright 2023 @ CAMEL-AI.org. All Rights Reserved. ===========
+# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
 
 from __future__ import annotations
 
@@ -20,10 +20,16 @@ from io import BytesIO
 from math import ceil
 from typing import TYPE_CHECKING, List, Optional
 
-from anthropic import Anthropic
 from PIL import Image
 
-from camel.types import ModelType, OpenAIImageType, OpenAIVisionDetailType
+from camel.logger import get_logger
+from camel.types import (
+    ModelType,
+    OpenAIImageType,
+    OpenAIVisionDetailType,
+    UnifiedModelType,
+)
+from camel.utils import dependencies_required
 
 if TYPE_CHECKING:
     from mistral_common.protocol.instruct.request import (  # type:ignore[import-not-found]
@@ -39,144 +45,7 @@ SQUARE_PIXELS = 512
 SQUARE_TOKENS = 170
 EXTRA_TOKENS = 85
 
-
-def messages_to_prompt(messages: List[OpenAIMessage], model: ModelType) -> str:
-    r"""Parse the message list into a single prompt following model-specific
-    formats.
-
-    Args:
-        messages (List[OpenAIMessage]): Message list with the chat history
-            in OpenAI API format.
-        model (ModelType): Model type for which messages will be parsed.
-
-    Returns:
-        str: A single prompt summarizing all the messages.
-    """
-    system_message = messages[0]["content"]
-
-    ret: str
-    if model in [
-        ModelType.LLAMA_2,
-        ModelType.LLAMA_3,
-        ModelType.GROQ_LLAMA_3_8B,
-        ModelType.GROQ_LLAMA_3_70B,
-    ]:
-        # reference: https://github.com/facebookresearch/llama/blob/cfc3fc8c1968d390eb830e65c63865e980873a06/llama/generation.py#L212
-        seps = [" ", " </s><s>"]
-        role_map = {"user": "[INST]", "assistant": "[/INST]"}
-
-        system_prompt = f"[INST] <<SYS>>\n{system_message}\n<</SYS>>\n\n"
-        ret = ""
-        for i, msg in enumerate(messages[1:]):
-            role = role_map[msg["role"]]
-            content = msg["content"]
-            if content:
-                if not isinstance(content, str):
-                    raise ValueError(
-                        "Currently multimodal context is not "
-                        "supported by the token counter."
-                    )
-                if i == 0:
-                    ret += system_prompt + content
-                else:
-                    ret += role + " " + content + seps[i % 2]
-            else:
-                ret += role
-        return ret
-    elif model in [ModelType.VICUNA, ModelType.VICUNA_16K]:
-        seps = [" ", "</s>"]
-        role_map = {"user": "USER", "assistant": "ASSISTANT"}
-
-        system_prompt = f"{system_message}"
-        ret = system_prompt + seps[0]
-        for i, msg in enumerate(messages[1:]):
-            role = role_map[msg["role"]]
-            content = msg["content"]
-            if not isinstance(content, str):
-                raise ValueError(
-                    "Currently multimodal context is not "
-                    "supported by the token counter."
-                )
-            if content:
-                ret += role + ": " + content + seps[i % 2]
-            else:
-                ret += role + ":"
-        return ret
-    elif model == ModelType.GLM_4_OPEN_SOURCE:
-        system_prompt = f"[gMASK]<sop><|system|>\n{system_message}"
-        ret = system_prompt
-        for msg in messages[1:]:
-            role = msg["role"]
-            content = msg["content"]
-            if not isinstance(content, str):
-                raise ValueError(
-                    "Currently multimodal context is not "
-                    "supported by the token counter."
-                )
-            if content:
-                ret += "<|" + role + "|>" + "\n" + content
-            else:
-                ret += "<|" + role + "|>" + "\n"
-        return ret
-    elif model == ModelType.QWEN_2:
-        system_prompt = f"<|im_start|>system\n{system_message}<|im_end|>"
-        ret = system_prompt + "\n"
-        for msg in messages[1:]:
-            role = msg["role"]
-            content = msg["content"]
-            if not isinstance(content, str):
-                raise ValueError(
-                    "Currently multimodal context is not "
-                    "supported by the token counter."
-                )
-            if content:
-                ret += (
-                    '<|im_start|>'
-                    + role
-                    + '\n'
-                    + content
-                    + '<|im_end|>'
-                    + '\n'
-                )
-            else:
-                ret += '<|im_start|>' + role + '\n'
-        return ret
-    elif model == ModelType.GROQ_MIXTRAL_8_7B:
-        # Mistral/Mixtral format
-        system_prompt = f"<s>[INST] {system_message} [/INST]\n"
-        ret = system_prompt
-
-        for msg in messages[1:]:
-            if msg["role"] == "user":
-                ret += f"[INST] {msg['content']} [/INST]\n"
-            elif msg["role"] == "assistant":
-                ret += f"{msg['content']}</s>\n"
-
-            if not isinstance(msg['content'], str):
-                raise ValueError(
-                    "Currently multimodal context is not "
-                    "supported by the token counter."
-                )
-
-        return ret.strip()
-    elif model in [ModelType.GROQ_GEMMA_7B_IT, ModelType.GROQ_GEMMA_2_9B_IT]:
-        # Gemma format
-        ret = f"<bos>{system_message}\n"
-        for msg in messages:
-            if msg["role"] == "user":
-                ret += f"Human: {msg['content']}\n"
-            elif msg["role"] == "assistant":
-                ret += f"Assistant: {msg['content']}\n"
-
-            if not isinstance(msg['content'], str):
-                raise ValueError(
-                    "Currently multimodal context is not supported by the token counter."
-                )
-
-        ret += "<eos>"
-        return ret
-    else:
-        raise ValueError(f"Invalid model type: {model}")
+logger = get_logger(__name__)
 
 
 def get_model_encoding(value_for_tiktoken: str):
@@ -193,8 +62,15 @@ def get_model_encoding(value_for_tiktoken: str):
     try:
         encoding = tiktoken.encoding_for_model(value_for_tiktoken)
     except KeyError:
-        print("Model not found. Using cl100k_base encoding.")
-        encoding = tiktoken.get_encoding("cl100k_base")
+        if value_for_tiktoken in [
+            ModelType.O1.value,
+            ModelType.O1_MINI.value,
+            ModelType.O1_PREVIEW.value,
+        ]:
+            encoding = tiktoken.get_encoding("o200k_base")
+        else:
+            logger.info("Model not found. Using cl100k_base encoding.")
+            encoding = tiktoken.get_encoding("cl100k_base")
     return encoding
 
 
@@ -215,67 +91,15 @@ class BaseTokenCounter(ABC):
         pass
 
 
-class OpenSourceTokenCounter(BaseTokenCounter):
-    def __init__(self, model_type: ModelType, model_path: str):
-        r"""Constructor for the token counter for open-source models.
-
-        Args:
-            model_type (ModelType): Model type for which tokens will be
-                counted.
-            model_path (str): The path to the model files, where the tokenizer
-                model should be located.
-        """
-
-        # Use a fast Rust-based tokenizer if it is supported for a given model.
-        # If a fast tokenizer is not available for a given model,
-        # a normal Python-based tokenizer is returned instead.
-        from transformers import AutoTokenizer
-
-        try:
-            tokenizer = AutoTokenizer.from_pretrained(
-                model_path,
-                use_fast=True,
-            )
-        except TypeError:
-            tokenizer = AutoTokenizer.from_pretrained(
-                model_path,
-                use_fast=False,
-            )
-        except Exception:
-            raise ValueError(
-                f"Invalid `model_path` ({model_path}) is provided. "
-                "Tokenizer loading failed."
-            )
-
-        self.tokenizer = tokenizer
-        self.model_type = model_type
-
-    def count_tokens_from_messages(self, messages: List[OpenAIMessage]) -> int:
-        r"""Count number of tokens in the provided message list using
-        loaded tokenizer specific for this type of model.
-
-        Args:
-            messages (List[OpenAIMessage]): Message list with the chat history
-                in OpenAI API format.
-
-        Returns:
-            int: Number of tokens in the messages.
-        """
-        prompt = messages_to_prompt(messages, self.model_type)
-        input_ids = self.tokenizer(prompt).input_ids
-
-        return len(input_ids)
-
-
 class OpenAITokenCounter(BaseTokenCounter):
-    def __init__(self, model: ModelType):
+    def __init__(self, model: UnifiedModelType):
         r"""Constructor for the token counter for OpenAI models.
 
         Args:
-            model (ModelType): Model type for which tokens will be counted.
+            model (UnifiedModelType): Model type for which tokens will be
+                counted.
         """
         self.model: str = model.value_for_tiktoken
-        self.model_type = model
 
         self.tokens_per_message: int
         self.tokens_per_name: int
@@ -288,6 +112,9 @@ class OpenAITokenCounter(BaseTokenCounter):
         elif ("gpt-3.5-turbo" in self.model) or ("gpt-4" in self.model):
             self.tokens_per_message = 3
             self.tokens_per_name = 1
+        elif "o1" in self.model:
+            self.tokens_per_message = 2
+            self.tokens_per_name = 1
         else:
             # flake8: noqa :E501
             raise NotImplementedError(
@@ -318,12 +145,19 @@ class OpenAITokenCounter(BaseTokenCounter):
             num_tokens += self.tokens_per_message
             for key, value in message.items():
                 if not isinstance(value, list):
-                    num_tokens += len(self.encoding.encode(str(value)))
+                    num_tokens += len(
+                        self.encoding.encode(str(value), disallowed_special=())
+                    )
                 else:
                     for item in value:
                         if item["type"] == "text":
                             num_tokens += len(
-                                self.encoding.encode(str(item["text"]))
+                                self.encoding.encode(
+                                    str(
+                                        item["text"],
+                                    ),
+                                    disallowed_special=(),
+                                )
                             )
                         elif item["type"] == "image_url":
                             image_str: str = item["image_url"]["url"]
@@ -395,18 +229,19 @@ class OpenAITokenCounter(BaseTokenCounter):
 
 
 class AnthropicTokenCounter(BaseTokenCounter):
-    def __init__(self, model_type: ModelType):
+    @dependencies_required('anthropic')
+    def __init__(self, model: str):
         r"""Constructor for the token counter for Anthropic models.
 
         Args:
-            model_type (ModelType): Model type for which tokens will be
-                counted.
+            model (str): The name of the Anthropic model being used.
         """
+        from anthropic import Anthropic
 
-        self.model_type = model_type
         self.client = Anthropic()
-        self.tokenizer = self.client.get_tokenizer()
+        self.model = model
 
+    @dependencies_required('anthropic')
     def count_tokens_from_messages(self, messages: List[OpenAIMessage]) -> int:
         r"""Count number of tokens in the provided message list using
         loaded tokenizer specific for this type of model.
@@ -418,20 +253,31 @@ class AnthropicTokenCounter(BaseTokenCounter):
         Returns:
             int: Number of tokens in the messages.
         """
-        num_tokens = 0
-        for message in messages:
-            content = str(message["content"])
-            num_tokens += self.client.count_tokens(content)
-        return num_tokens
+        from anthropic.types import MessageParam
+
+        return self.client.messages.count_tokens(
+            messages=[
+                MessageParam(
+                    content=str(msg["content"]),
+                    role="user" if msg["role"] == "user" else "assistant",
+                )
+                for msg in messages
+            ],
+            model=self.model,
+        ).input_tokens
 
 
 class GeminiTokenCounter(BaseTokenCounter):
-    def __init__(self, model_type: ModelType):
-        r"""Constructor for the token counter for Gemini models."""
+    def __init__(self, model_type: UnifiedModelType):
+        r"""Constructor for the token counter for Gemini models.
+
+        Args:
+            model_type (UnifiedModelType): Model type for which tokens will be
+                counted.
+        """
         import google.generativeai as genai
 
-        self.model_type = model_type
-        self._client = genai.GenerativeModel(self.model_type.value)
+        self._client = genai.GenerativeModel(model_type)
 
     def count_tokens_from_messages(self, messages: List[OpenAIMessage]) -> int:
         r"""Count number of tokens in the provided message list using
@@ -459,12 +305,13 @@ class GeminiTokenCounter(BaseTokenCounter):
         return self._client.count_tokens(converted_messages).total_tokens
 
 
-class LiteLLMTokenCounter:
-    def __init__(self, model_type: str):
+class LiteLLMTokenCounter(BaseTokenCounter):
+    def __init__(self, model_type: UnifiedModelType):
         r"""Constructor for the token counter for LiteLLM models.
 
         Args:
-            model_type (str): Model type for which tokens will be counted.
+            model_type (UnifiedModelType): Model type for which tokens will be
+                counted.
         """
         self.model_type = model_type
         self._token_counter = None
@@ -529,8 +376,11 @@ class MistralTokenCounter(BaseTokenCounter):
         model_name = (
             "codestral-22b"
             if self.model_type
-            in {ModelType.MISTRAL_CODESTRAL, ModelType.MISTRAL_CODESTRAL_MAMBA}
-            else self.model_type.value
+            in {
+                ModelType.MISTRAL_CODESTRAL,
+                ModelType.MISTRAL_CODESTRAL_MAMBA,
+            }
+            else self.model_type
         )
 
         self.tokenizer = MistralTokenizer.from_model(model_name)
@@ -573,7 +423,7 @@ class MistralTokenCounter(BaseTokenCounter):
         )
 
         mistral_request = ChatCompletionRequest(  # type: ignore[type-var]
-            model=self.model_type.value,
+            model=self.model_type,
             messages=[openai_msg],
         )
 
